<?php

namespace Drupal\spectra_expire\Plugin\Spectra\ExpirationMethod;

use Drupal\Core\Plugin\PluginBase;
use Drupal\spectra_expire\SpectraExpirationMethodInterface;
use Drupal\spectra\Entity\SpectraData;
use Drupal\spectra\Entity\SpectraStatement;

/**
 *
 * @SpectraExpirationMethod(
 *   id = "delete_statements",
 *   select_options = {
 *     "spectra_expire_all_statements" = @Translation("All Statements"),
 *     "spectra_expire_statement_type" = @Translation("Statements of a given type"),
 *     "spectra_expire_data_type" = @Translation("Statements with data of a given type"),
 *   },
 *   expire_options = {
 *     "spectra_expire_delete_statements_and_data" = @Translation("Delete Statements and associated data, but leave other entities alone"),
 *     "spectra_expire_delete_all" = @Translation("Delete associated actors, actions, etc. if no other statements reference it"),
 *   },
 *   label = @Translation("DefaultSpectraExpirationMethod"),
 * )
 */
class DefaultSpectraExpirationMethod extends PluginBase implements SpectraExpirationMethodInterface {

  /**
   * @return string
   *   A string description of the plugin.
   */
  public function description()
  {
    return $this->t('Delete Spectra Statements');
  }


  /**
   * @inheritdoc
   *
   * Finds Expiring Spectra Statements
   *
   * @param (array) statement_list:
   *   Array of statement IDs
   *
   * @see \Drupal::entityQuery()
   */
  public static function select_expiring_statements($method, $policy) {
    $expire_time = time() - (isset($policy->get('expire_age')->getValue()[0]['value']) ? $policy->get('expire_age')->getValue()[0]['value']: 0);
    $entity_type = isset($policy->get('entity_type')->getValue()[0]['value']) ? $policy->get('entity_type')->getValue()[0]['value']: FALSE;
    $expire_max = isset($policy->get('expire_max')->getValue()[0]['value']) ? $policy->get('expire_max')->getValue()[0]['value']: 0;

    $statements = array();
    switch ($method) {
      case 'spectra_expire_all_statements':
      case 'spectra_expire_statement_type':
        $query = \Drupal::entityQuery('spectra_statement');
        $query->condition('created', $expire_time, '<');
        if ($entity_type && $method === 'spectra_expire_statement_type') {
          $query->condition('statement_type', $entity_type, '<');
        }
        $query->pager($expire_max);
        $query->sort('statement_id', 'ASC');
        $result = array_keys($query->execute());

        $statements = SpectraStatement::loadMultiple($result);
        break;
      case 'spectra_expire_data_type':
        $data_query = \Drupal::entityQuery('spectra_data');
        $data_query->condition('created', $expire_time, '<');
        if ($entity_type) {
          $data_query->condition('data_type', $entity_type, '<');
        }
        $data_query->pager($expire_max);
        $data_query->sort('data_id', 'ASC');
        $result = array_keys($data_query->execute());
        $data = SpectraStatement::loadMultiple($result);
        $statement_array = array();
        foreach ($data as $d) {
          if (isset($d->get('statement_id')->getValue()[0]['target_id'])) {
            $id = $d->get('statement_id')->getValue()[0]['target_id'];
            if(!isset($statement_array[$id])) {
              $statement_array[$id] = SpectraStatement::load($id);
            }
          }
        }
        break;
    }
    return $statements;
  }

  /**
   * @inheritdoc
   *
   * Expires Spectra Statements
   *
   * @param (array) statement_list:
   *   Array of statement IDs
   *
   * @see \Drupal::entityQuery()
   */
  public function expire_statements($method, $statement_list) {
    switch ($method) {
      case 'spectra_expire_delete_statements_and_data':
        foreach($statement_list as $statement) {
          $statement->delete();
        }
        break;
      case 'spectra_expire_delete_all':
        foreach($statement_list as $statement) {
          $statement->deleteAssociatedEntities();
          $statement->delete();
        }
        break;
    }
  }



}